@@ -2,10 +2,10 @@
 from __future__ import annotations
 
 import asyncio
-from datetime import datetime, timedelta
+from datetime import datetime
 from json.decoder import JSONDecodeError
 import sys
-from typing import TYPE_CHECKING, Any, Callable
+from typing import TYPE_CHECKING, Any, Callable, cast
 
 from aiohttp import ClientSession
 from aiohttp.client_exceptions import ClientResponseError
@@ -31,19 +31,9 @@ from simplipy.websocket import WebsocketClient
 API_URL_HOSTNAME = "api.simplisafe.com"
 API_URL_BASE = f"https://{API_URL_HOSTNAME}/v1"
 
-DEFAULT_EXPIRATION_PADDING = 300
 DEFAULT_REQUEST_RETRIES = 4
 DEFAULT_TIMEOUT = 10
-
-
-def get_expiration_datetime(expires_in_seconds: int) -> datetime:
-    """Get a token expiration datetime as an offset of UTC now + a number of seconds.
-
-    Note that we pad the value to ensure the token doesn't expire without us knowing.
-    """
-    return datetime.utcnow() + (
-        timedelta(seconds=expires_in_seconds - DEFAULT_EXPIRATION_PADDING)
-    )
+DEFAULT_TOKEN_EXPIRATION_WINDOW = 5
 
 
 class API:  # pylint: disable=too-many-instance-attributes
@@ -67,27 +57,19 @@ class API:  # pylint: disable=too-many-instance-attributes
     ) -> None:
         """Initialize."""
         self._refresh_token_callbacks: list[Callable[..., None]] = []
+        self._request_retries = request_retries
         self.session: ClientSession = session
 
         # These will get filled in after initial authentication:
-        self._access_token_expire_dt: datetime | None = None
         self._backoff_refresh_lock = asyncio.Lock()
+        self._token_last_refreshed: datetime | None = None
         self.access_token: str | None = None
         self.refresh_token: str | None = None
         self.subscription_data: dict[int, Any] = {}
         self.user_id: int | None = None
         self.websocket: WebsocketClient | None = None
 
-        # Implement a version of the request coroutine, but with backoff/retry logic:
-        self.request = backoff.on_exception(
-            backoff.expo,
-            ClientResponseError,
-            jitter=backoff.random_jitter,
-            logger=LOGGER,
-            max_tries=request_retries,
-            on_backoff=self._async_handle_on_backoff,
-            on_giveup=self._async_handle_on_giveup,
-        )(self._async_request)
+        self.async_request = self._wrap_request_method(self._request_retries)
 
     @classmethod
     async def async_from_auth(
@@ -127,11 +109,11 @@ class API:  # pylint: disable=too-many-instance-attributes
                 },
             )
         except ClientResponseError as err:
-            if err.status == 401 or err.status == 403:
+            if err.status in (401, 403):
                 raise InvalidCredentialsError("Invalid credentials") from err
             raise RequestError(err) from err
 
-        api._access_token_expire_dt = get_expiration_datetime(token_resp["expires_in"])
+        api._token_last_refreshed = datetime.utcnow()
         api.access_token = token_resp["access_token"]
         api.refresh_token = token_resp["refresh_token"]
         await api._async_post_init()
@@ -166,27 +148,28 @@ class API:  # pylint: disable=too-many-instance-attributes
         err_info = sys.exc_info()
         err: ClientResponseError = err_info[1].with_traceback(err_info[2])  # type: ignore
 
-        if err.status == 401 or err.status == 403:
+        LOGGER.debug("Error during request attempt: %s", err)
+
+        if err.status in (401, 403):
             if TYPE_CHECKING:
-                assert self._access_token_expire_dt
+                assert self._token_last_refreshed
+
+            # Calculate the window between now and the last time the token was
+            # refreshed:
+            window = (datetime.utcnow() - self._token_last_refreshed).total_seconds()
 
             # Since we might have multiple requests (each running their own retry
             # sequence) land here, we only refresh the access token if it hasn't
-            # been refreshed within the expiration window (and we lock the attempt so
-            # other requests can't try it at the same time):
+            # been refreshed within the window (and we lock the attempt so other
+            # requests can't try it at the same time):
             async with self._backoff_refresh_lock:
-                if datetime.utcnow() < self._access_token_expire_dt:
+                if window < DEFAULT_TOKEN_EXPIRATION_WINDOW:
+                    LOGGER.debug("Skipping refresh attempt since window hasn't busted")
                     return
 
                 LOGGER.info("401 detected; attempting refresh token")
                 await self._async_refresh_access_token()
 
-    async def _async_handle_on_giveup(self, _: dict[str, Any]) -> None:
-        """Handle a give up after retries are exhausted."""
-        err_info = sys.exc_info()
-        err = err_info[1].with_traceback(err_info[2])  # type: ignore
-        raise RequestError(err) from err
-
     async def _async_post_init(self) -> None:
         """Perform some post-init actions."""
         auth_check_resp = await self._async_request("get", "api/authCheck")
@@ -208,11 +191,11 @@ class API:  # pylint: disable=too-many-instance-attributes
                 },
             )
         except ClientResponseError as err:
-            if err.status == 401 or err.status == 403:
+            if err.status in (401, 403):
                 raise InvalidCredentialsError("Invalid refresh token") from err
             raise RequestError(err) from err
 
-        self._access_token_expire_dt = get_expiration_datetime(token_resp["expires_in"])
+        self._token_last_refreshed = datetime.utcnow()
         self.access_token = token_resp["access_token"]
         self.refresh_token = token_resp["refresh_token"]
 
@@ -262,6 +245,36 @@ class API:  # pylint: disable=too-many-instance-attributes
 
         return data
 
+    @staticmethod
+    def _handle_on_giveup(_: dict[str, Any]) -> None:
+        """Handle a give up after retries are exhausted."""
+        err_info = sys.exc_info()
+        err = err_info[1].with_traceback(err_info[2])  # type: ignore
+        raise RequestError(err) from err
+
+    def _wrap_request_method(self, request_retries: int) -> Callable:
+        """Wrap the request method in backoff/retry logic."""
+        return cast(
+            Callable,
+            backoff.on_exception(
+                backoff.expo,
+                ClientResponseError,
+                jitter=backoff.random_jitter,
+                logger=LOGGER,
+                max_tries=request_retries,
+                on_backoff=self._async_handle_on_backoff,
+                on_giveup=self._handle_on_giveup,
+            )(self._async_request),
+        )
+
+    def disable_request_retries(self) -> None:
+        """Disable the request retry mechanism."""
+        self.async_request = self._wrap_request_method(1)
+
+    def enable_request_retries(self) -> None:
+        """Enable the request retry mechanism."""
+        self.async_request = self._wrap_request_method(self._request_retries)
+
     def add_refresh_token_callback(
         self, callback: Callable[..., None]
     ) -> Callable[..., None]:
@@ -318,7 +331,7 @@ class API:  # pylint: disable=too-many-instance-attributes
 
     async def async_update_subscription_data(self) -> None:
         """Get the latest subscription data."""
-        subscription_resp = await self.request(
+        subscription_resp = await self.async_request(
             "get", f"users/{self.user_id}/subscriptions", params={"activeOnly": "true"}
         )
         self.subscription_data = {
